@@ -47,6 +47,7 @@
 #define VERSION    1                         /* current version number */
 
 void haval_string (char *, unsigned char *); /* hash a string */
+void haval_string_n (char *, unsigned char *, unsigned int);
 int  haval_file (char *, unsigned char *);   /* hash a file */
 void haval_stdin (void);                     /* hash input from stdin */
 void haval_start (haval_state *);            /* initialization */
@@ -228,6 +229,16 @@ void haval_string (char *string, unsigned char fingerprint[FPTLEN >> 3])
   haval_end (&state, fingerprint);
 }
 
+/* hash a string */
+void haval_string_n (char *string, unsigned char fingerprint[FPTLEN >> 3], unsigned int n)
+{
+  haval_state   state;
+
+  haval_start (&state);
+  haval_hash (&state, (unsigned char *)string, n);
+  haval_end (&state, fingerprint);
+}
+
 /* hash a file */
 int haval_file (char *file_name, unsigned char fingerprint[FPTLEN >> 3])
 {
