@@ -23,7 +23,7 @@ char rbuf[2048];
     printf("%s = %s\n", label,rbuf); }
 #endif
 
-unsigned char ver[] = "@(#)L3 1.7.5 - Mark C. Henderson - markh@wimsey.bc.ca";
+unsigned char ver[] = "@(#)L3 2.0 - Mark C. Henderson - markh@wimsey.bc.ca";
 
 #ifndef STAND
 extern void ferr(char *);
@@ -780,6 +780,7 @@ void private_from_primes(MP_INT * res, MP_INT * pub, MP_INT * p, MP_INT * q)
 	modinv(res, pub, res);
 }
 
+#if 0
 /*
  * Lucas function. Calculates V_e(p, 1) mod n
  * We use the following identities for V_n(P,1)
@@ -864,6 +865,72 @@ void v_p1_n(MP_INT * x, MP_INT * ec, MP_INT * p, MP_INT * n)
 	clear(x2);
 	clear(t);
 }
+#else
+/*
+ * Lucas function. Calculates V_e(p, 1) mod n
+ * We use the following identities for V_n(P,1)
+ *  V_{2n} = (V_n)^2 - 2
+ *  V_{2n+1} = V_n*V{n-1} - P
+ * 
+ * thanks to Wei Dai <weidai@eskimo.com> for pointing this out
+ * I don't really know why I didn't see this simplification before
+ */
+void v_p1_n(MP_INT * x, MP_INT * ec, MP_INT * p, MP_INT * n)
+{
+	struct is *stack = NULL;
+	MP_INT *e, *y;
+	int k,i;
+
+	if (!mpz_cmp_ui(ec, 0)) {
+		mpz_set_ui(x, 2);
+		return;
+	}
+	init(e); init(y);
+
+	mpz_set(e, ec);	/* e is a copy of ec which we can munge */
+
+    /* push the bits of e onto a stack, starting at the rightmost (LSB) */
+	for (k = 0; mpz_cmp_ui(e, 0); k++, mpz_div_2exp(e, e, 1))
+		push(mpz_get_ui(e) & 1, &stack);
+	k--;
+
+	mpz_set(x,p);   /* x = p */
+
+	/* y = (p * p - 2) % n */
+	mpz_mul(y,p,p); mpz_sub_ui(y,y,2); mpz_mmod(y,y,n);
+
+	/*
+     * at this point all of the bits of e have been pushed. Now
+     * when we pop, we'll get them left-most (most significant) first,
+     * and right-most (least significant) last.
+     */
+
+	/* discard most significant bit */
+	i = pop(&stack);
+
+	for (i = k - 1; i >= 0; i--) {
+
+		if (pop(&stack)) {	
+
+			/* x = (x*y - p) % n */
+			mpz_mul(x,x,y); mpz_sub(x,x,p); mpz_mmod(x,x,n);
+
+			/* y = (y*y - 2) % n; */
+			mpz_mul(y,y,y); mpz_sub_ui(y,y,2); mpz_mmod(y,y,n);
+
+		}
+		else 
+		{
+			/* y = (x*y - p) % n */
+			mpz_mul(y,x,y); mpz_sub(y,y,p); mpz_mmod(y,y,n);
+
+			/* x = (x*x - 2) % n */
+			mpz_mul(x,x,x); mpz_sub_ui(x,x,2); mpz_mmod(x,x,n);
+		}
+	}
+	clear(e); clear(y);
+}
+#endif
 
 /*
  * calculate V_e(p,q) mod n
