@@ -13,6 +13,7 @@
 #include <signal.h>
 
 static char sname[1024];
+static rc4_key rc4key;
 static struct termios tty_orig, tty_new;
 static FILE *tty;
 extern int debug;
@@ -1770,6 +1771,11 @@ void initrandom()
 		seeddir("/tmp");
 		seeddir(".");
 #endif
+		for (i=0;i<10;i++) {
+			int j;
+			j = truerand();
+			seedin(&j,sizeof(j));
+		}
 	}
 }
 
@@ -1872,6 +1878,7 @@ unsigned char *getstring(int pwd)
 		return NULL;
 }
 
+/* more amateur theatrics */
 unsigned char randombyte()
 {
 	static int firstcall = 1;
@@ -1886,26 +1893,41 @@ unsigned char randombyte()
 	if (firstcall) {	/* we've got stuff to do */
 		MD5_CTX mdContext;
 		firstcall = 0;
+		/* b1 = MD5(rbuf) */
 		MD5Init(&mdContext);
 		MD5Update(&mdContext, (unsigned char *) rbuf, j);
 		MD5Final(&mdContext);
 		memcpy(b1, mdContext.digest, 16);
+
+		/* rc4key = rbuf */
+		prepare_key(rbuf,j,&rc4key);
+
+		/* des2key = b1 */
 		des2key(b1, 0);
+
+		/* b2 and b5 split the output of sn512string on rbuf */
 		sn512string(rbuf, j, h);
 		memcpy(b2, h, 16);
 		memcpy(b5, &h[16], 16);
 
 		zap(h, 32);	/* no reason to leave this lying around */
 
-		/* our state consists of b1 and b2 and b5,
+		/* our state consists of b1, b4, b2, b5.
          * b3 will be output */
 
+		/* b3 = D2des(b2). This makes the actual output */
 		D2des(&b2[0], &b3[0]);
+
+		/* b4 = idea with key of b5 (b1) with swaps */
 		initkey_idea(b5, 0);
 		idea_ecb((word16 *) & b1[0], (word16 *) & b4[8]);
 		idea_ecb((word16 *) & b1[8], (word16 *) & b4[0]);
+
+		/* b3 ^= b4 */
 		for (i = 0; i < 16; i++)
 			b3[i] ^= b4[i];
+
+		/* remember b1 was our deskey */
 		D2des(&b2[0], &b2[0]);
 		D2des(&b5[0], &b5[0]);
 
@@ -1916,19 +1938,43 @@ unsigned char randombyte()
 	}
 	if (used == 16) {
 
-		for (i = 0; i < 16; i++)
-			b1[i] = (b1[15 - i] + 1) % 0xff;
+		/* munge b1 and b5 */
+		rc4(b1,16,&rc4key);
+		rc4(b5,16,&rc4key);
 
+		b1[0] = b1[1] + 7;
+		b1[1] =  b1[2] + 11;
+		b1[2] = b1[3] + 3;
+		b1[3] = b1[4] + b2[5]  + 5;
+		b1[4] = b1[5] + 17;
+		b1[5] = b1[6] + 19;
+		b1[6] = b1[7] + 13;
+		b1[7] = b1[0] + b3[1] + 23;
+
+		for (i=0;i<8;i++)
+			b1[i]+=7;
+
+		/* now let b1 be our new des2key */
 		des2key(b1, 0);
+
+		/* this makes the first part of the stuff that gets xord 
+           into the actual output */
 		D2des(&b2[0], &b3[0]);
+
+		/* set b4 according to b5 and b1 */
 		initkey_idea(b5, 0);
 		idea_ecb((word16 *) & b1[0], (word16 *) & b4[8]);
 		idea_ecb((word16 *) & b1[8], (word16 *) & b4[0]);
+		
+		/* munge output according to b4 */
 		for (i = 0; i < 16; i++)
 			b3[i] ^= b4[i];
+		
 
+		/* munge b2 and b5 */
 		D2des(&b2[0], &b2[0]);
 		D2des(&b5[0], &b5[0]);
+
 		used = 0;
 		close_des();
 		close_idea();
@@ -1972,6 +2018,7 @@ void closerandom()
 			b1[i] = b2[i] = b3[i] = b4[i] = b5[i] = 0;
 		}
 		zap(rbuf, RBUFLEN);
+		zap(&rc4key, sizeof(rc4key));
 	}
 }
 
