@@ -52,7 +52,7 @@ main(int argc, char *argv[])
 	fputc((char) alg, stdout);
 	write_number(stdout, c);
 	zap(k, 16);
-	zap(hash, 16);
+	zap(hash, HASHLEN);
 	zap(ss, strlen(ss));
 	exit(0);
 }
