@@ -18,5 +18,5 @@ echo you should see no differences here
 diff foo ssk.c
 read i
 echo checking signature
-$PREFIX/vsign markh@wimsey.bc.ca MANIFEST.sig <MANIFEST
+$PREFIX/vsign markh@wimsey.bc.ca MANIFEST.lsig <MANIFEST
 echo done
